#pragma once

/////////////////////////////////////////////////////////////////////
// Export all classes and functions for DLL version
/////////////////////////////////////////////////////////////////////
/*
   For writing new parser you must implement IParser and IExtensions
local interfaces and two global functions in your DLL. Local iterfaces
look like COM interfaces but are not registered and they use a string as
an indentificator. In this case if you want to write a parser for *.cpp
files use "cpp" as the indentificator. So, TreeView call
CreateInterface("cpp", 0, ...) and get your parser for *.cpp files.
You can bind some extensions with one parser. A new DLL must be placed
to the TreeView folder and will be loaded automatically by TreeView at start.

   Order = {1..n, 1} from 1 to n step 1 with no exceptions
	Image = {1..n, 1} every item type (or in other words order or folder) MUST have ONE image
*/


// Local interfaces result codes
#define LI_OK						((HRESULT)0x00000000)
#define LI_FALSE					((HRESULT)0x00000001)

#define LI_FAIL					((HRESULT)0x80000001)

// Parser error result codes
#define PE_START					((HRESULT)0x80000002)
#define PE_INCLUDETAG			((HRESULT)0x80000003)
#define PE_TAG						((HRESULT)0x80000004)
#define PE_FINISH					((HRESULT)0x80000005)

#define LI_SUCCEEDED(Status)	((HRESULT)(Status) >= 0)
#define LI_FAILED(Status)		((HRESULT)(Status) < 0)

class IParser;

class ILocalUnknown
{
public:
	virtual UINT AddRef() = 0;
	virtual HRESULT Query(LPCTSTR czIdentifier, UINT uiVersion, void** ppObject) = 0;
	virtual UINT Release() = 0;
	virtual HRESULT Enum(int* piInterface, int* piIdentifier, LPCTSTR* pczIdentifier, UINT* puiVersion) = 0;
	virtual UINT GetVersion() = 0;
};

class ICallbackParser : public ILocalUnknown
{
public:
	virtual HRESULT Start(IParser* pIParser, LPCTSTR czFileName, int iImageOffset, int iFolderCount) = 0;
	virtual HRESULT ProcessIncludeTag(LPCTSTR czTagName, LPCTSTR czFileName, DWORD fType, int iOrder, int iImage, int iColumn, int iRow) = 0;
	virtual HRESULT ProcessTag(LPCTSTR czTagName, LPCTSTR czIncludeFileName, int iOrder, int iImage, int iColumn, int iRow) = 0;
	virtual HRESULT Finish() = 0;
	virtual HRESULT IsShown(int iOrder, BOOL* pbRes) = 0;
};

class IExtensions : public ILocalUnknown
{
public:
	virtual HRESULT Load() = 0;
	virtual HRESULT Save() = 0;

	virtual HRESULT IsExtension(LPCTSTR czExtension) = 0;
	virtual HRESULT GetExtensionsCount(int* piCount) = 0;
	virtual HRESULT GetExtension(int iIndex, LPCTSTR* pczExtension) = 0;
	virtual HRESULT DefaultExtensions() = 0;
	virtual HRESULT ClearExtensions() = 0;
	virtual HRESULT AddExtension(LPCTSTR czExtension) = 0;

	virtual HRESULT GetExtFilterTitle(LPTSTR czExtFilterTitle, LPDWORD pdwSize) = 0;
	virtual HRESULT GetExtFilter(LPTSTR czExtFilter, LPDWORD pdwSize) = 0;
};

#define P_DEFAULT					P_CREATE_FOLDERS
#define P_CREATE_FOLDERS		0x00000001

class IParser : public ILocalUnknown
{
public:
	virtual HRESULT Parse(LPCTSTR czFileName, ICallbackParser* pICallbackParser, DWORD dwType = P_DEFAULT) = 0;
	virtual HRESULT Parse(LPVOID pBuffer, DWORD dwSize, ICallbackParser* pICallbackParser, DWORD dwType = P_DEFAULT) = 0;

	virtual HRESULT BeginStringParsing(LPCTSTR czFileName, ICallbackParser* pICallbackParser, DWORD dwType = P_DEFAULT) = 0;
	virtual HRESULT ParseString(LPCTSTR czString, int iAddLines) = 0;
	virtual HRESULT EndStringParsing() = 0;

	// File extensions (start)
	virtual HRESULT CreateExtensionsObject(IExtensions** ppIExtensions) = 0;

	virtual HRESULT IsExtension(LPCTSTR czExtension) = 0;
	virtual HRESULT GetExtensionsCount(int* piCount) = 0;
	virtual HRESULT GetExtension(int iIndex, LPCTSTR* pczExtension) = 0;
	virtual HRESULT DefaultExtensions() = 0;
	virtual HRESULT ClearExtensions() = 0;
	virtual HRESULT AddExtension(LPCTSTR czExtension) = 0;

	virtual HRESULT GetExtFilterTitle(LPTSTR czExtFilterTitle, LPDWORD pdwSize) = 0;
	virtual HRESULT GetExtFilter(LPTSTR czExtFilter, LPDWORD pdwSize) = 0;
	// File extensions (end)

	// Parser options (start)
	virtual HRESULT GetOptionWindowsCount(int* piCount) = 0;
	virtual HRESULT GetOptionWindowName(int iIndex, LPCTSTR* pczWindowName) = 0;
	virtual HRESULT CreateOptionWindow(LPCTSTR czWindowName, HWND hParentWnd, HWND* phWnd) = 0;
	// Parser oprions (end)

	virtual HRESULT GetImageOffset(int* piImageOffset) = 0;

	virtual HRESULT GetFoldersCount(int* piCount) = 0;
	virtual HRESULT GetFolder(int iIndex, LPCTSTR* pczName, int* piOrder, int* piImage) = 0;

	virtual HRESULT IsParserName(LPCTSTR czParserName) = 0;
	virtual HRESULT GetParserName(LPCTSTR* pczParserName) = 0;
};

extern "C" HRESULT __stdcall CreateInterface(LPCTSTR czIdentifier, UINT uiVersion, void** ppObject);
extern "C" HRESULT __stdcall CanUnloadNow();
